//! \file    ConnectionWindow.cpp
//! \brief   Implementation of the Ctin window.
//! \version 2010/07/06 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

/*
  Nom du Fichier: ConnectionWindow.cpp
  Nom de classe: ConnectionWindow
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

//! \class ConnectionWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Ctin window for visual_YAO.

#include "include/ConnectionWindow.hpp"

using namespace std;
using namespace yao;

/************************************************* Implementation of the construtor *******************************************************************/
ConnectionWindow::ConnectionWindow(Table <Modul> *aModulTable, Table <Constant> *aConstantTable, QWidget *parent) : QWidget(parent), cpt(1) { /*!< The constructor. */
	setupUi(this);
    
	m_ctinModel	= new QStandardItemModel(0, 8);	

	m_localModulTable	= aModulTable;	
	m_localConstantTable	= aConstantTable;	
	
	m_ctinModulInCombo->insertItem(0, "");
	m_ctinModulOutCombo->insertItem(0, ""); 
               		
		

	m_ctinModel->setHorizontalHeaderLabels(QStringList() << "Entree" << "Nbre Entrees" << "Sortie" << "Nbre Sorties" << "i" << "j" << "k" << "t");

	m_ctinTableView->setModel(m_ctinModel);	
	m_ctinTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
	m_ctinTableView->setShowGrid(false);
	m_ctinTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
	m_ctinTableView->resizeColumnsToContents();
	
	for (Table<Modul>::reverse_iterator it = m_localModulTable->rbegin(); it != m_localModulTable->rend(); it++) 
		{
			//int row = m_ctinModel->rowCount();                          
                 	
			m_ctinModulInCombo->insertItem(1, QString(it->getName().c_str()));
			m_ctinModulOutCombo->insertItem(1, QString(it->getName().c_str())); 
               		//m_ctinModel->setItem(row, 0, new QStandardItem(QString(it->getName().c_str()))); /*!< Gets the name.*/
               		//m_ctinModel->setItem(row, 1, new QStandardItem(QString(it->getSpaceOrOperator().c_str()))); 
		}               

	m_ctinInCombo->setEditable(true);
	m_ctinInCombo->insertItem(0, "");

	m_ctinIn2Combo->insertItem(0, "");

	m_ctinIn3Combo->insertItem(0, "");

	m_ctinOutCombo->setEditable(true);
	m_ctinOutCombo->insertItem(0, "");

	m_ctinOut2Combo->insertItem(0, "");

	m_ctinOut3Combo->insertItem(0, "");

	m_ctinICombo->setEditable(true);
	m_ctinICombo->insertItem(0, "");
	m_ctinICombo->insertItem(1, "+1");
	m_ctinICombo->insertItem(2, "-1");
	m_ctinICombo->insertItem(3, "...");
	
	m_ctinJCombo->setEditable(true);
	m_ctinJCombo->insertItem(0, "");
	m_ctinJCombo->insertItem(1, "+1");
	m_ctinJCombo->insertItem(2, "-1");
	m_ctinJCombo->insertItem(3, "...");
	
	m_ctinKCombo->setEditable(true);
	m_ctinKCombo->insertItem(0, "");
	m_ctinKCombo->insertItem(1, "+1");
	m_ctinKCombo->insertItem(2, "-1");
	m_ctinKCombo->insertItem(3, "...");
	
	m_ctinTCombo->setEditable(true);
	m_ctinTCombo->insertItem(0, "");
	m_ctinTCombo->insertItem(1, "+1");
	m_ctinTCombo->insertItem(2, "-1");
	m_ctinTCombo->insertItem(3, "...");
	
	for (Table<Constant>::reverse_iterator it = m_localConstantTable->rbegin(); it != m_localConstantTable->rend(); it++) 
		{
			//int row = m_ctinModel->rowCount();                          
                 	
			m_ctinInCombo->insertItem(1, QString(it->getName().c_str()));
			m_ctinIn2Combo->insertItem(1, QString(it->getName().c_str()));
			m_ctinIn3Combo->insertItem(1, QString(it->getName().c_str()));

			m_ctinOutCombo->insertItem(1, QString(it->getName().c_str()));
			m_ctinOut2Combo->insertItem(1, QString(it->getName().c_str()));
			m_ctinOut3Combo->insertItem(1, QString(it->getName().c_str()));


		//	m_ctinModulInCombo->insertItem(1, QString(it->getName().c_str()));
		//	m_ctinModulOutCombo->insertItem(1, QString(it->getName().c_str())); 
               		//m_ctinModel->setItem(row, 0, new QStandardItem(QString(it->getName().c_str()))); /*!< Gets the name.*/
               		//m_ctinModel->setItem(row, 1, new QStandardItem(QString(it->getSpaceOrOperator().c_str()))); 
			++cpt;
		}               

	m_ctinInCombo->insertItem(cpt, "...");
	m_ctinOutCombo->insertItem(cpt, "...");

	connect(m_ctinAddButton, SIGNAL(clicked()), this, SLOT(addCtinRow())); /*!< This slot add the object in the view */ 
	connect(m_ctinDelButton, SIGNAL(clicked()), this, SLOT(deleteCtinRow()));  /*!< slot  for the delete the object selected in the view*/
	connect(m_ctinQuitButton, SIGNAL(clicked()), this, SLOT(close()));
	connect(m_ctinHelpButton, SIGNAL(clicked()), this, SLOT(ctinHelp()));   /*!< This slot opens */

	connect(m_ctinInCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(activateInIntervalle(QString )));   /*!< This slot opens */
	connect(m_ctinOutCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(activateOutIntervalle(QString )));   /*!< This slot opens */
	connect(m_ctinJCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(activateJIntervalle(QString )));   /*!< This slot opens */
	connect(m_ctinKCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(activateKIntervalle(QString )));   /*!< This slot opens */
	connect(m_ctinTCombo, SIGNAL(currentIndexChanged(QString)), this, SLOT(activateTIntervalle(QString )));   /*!< This slot opens */

	setFixedWidth(baseSize().width() + 612);
  }
/********************************************** Implementation of the addCtinRow() method  ************************************************************/
void ConnectionWindow::addCtinRow() {
	
	try
		{
		}

	catch (...)
		{
			QMessageBox::critical(this, "Erreur", "erreur dans la methode addCtinRow().");
		}
	

/*
 static int row = 0;
  int l = 0;
  QString texte = " from ";

QString comboline1 = comboBoxctin1->currentText();
QString comboline2 = comboBoxctin2->currentText();
QString comboline3 = comboBoxctin3->currentText();
QString comboline4 = comboBoxctin4->currentText();
//QString comboline5 = comboBoxctin1->currentText();
//QString comboline1 = comboBoxctin1->currentText();


             for(int i = 0; i<comboline1.size(); i++)
        //if (m_ligneNomTraj->text().isEmpty())
     if(!((comboline1.at(i) >= QChar('a')&& comboline1.at(i) <= QChar('z')) || (comboline1.at(i) >= QChar('A')&& comboline1.at(i) <= QChar('Z')) //--------
         || (comboline1.at(i) >= QChar('0')&& comboline1.at(i) <= QChar('9')) || (comboline1.at(i) == QChar('_')))) //|| (line1.isEmpty())
    {
        QMessageBox::critical(this, "Erreur", "Veuillez entrer une valeur correcte pour le module entre1");
        return; // Arrêt de la méthode
    }
       for(int i1 = 0; i1<comboline2.size(); i1++)
        //if (m_ligneNomTraj->text().isEmpty())
     if(!((comboline2.at(i1) >= QChar('0')&& comboline2.at(i1) <= QChar('9')))) //|| (line1.isEmpty())
    {
        QMessageBox::critical(this, "Erreur", "Veuillez entrer une valeur correcte pour le module entre 2");
        return; // Arrêt de la méthode
    }
       for(int i3 = 0; i3<comboline3.size(); i3++) 
     if(!((comboline3.at(i3) >= QChar('0')&& comboline3.at(i3) <= QChar('9')))) //||
    {
        QMessageBox::critical(this, "Erreur", "Veuillez entrer une valeur correcte pour le module sortie 2");
        return; // Arrêt de la méthode
    }
       for(int i4 = 0; i4<comboline4.size(); i4++)
     if(!((comboline4.at(i4) >= QChar('a')&& comboline4.at(i4) <= QChar('z')) || (comboline4.at(i4) >= QChar('A')&& comboline4.at(i4) <= QChar('Z'))
         || (comboline4.at(i4) >= QChar('0')&& comboline4.at(i4) <= QChar('9')) || (comboline4.at(i4) == QChar('_')))) //|| 
    {
        QMessageBox::critical(this, "Erreur", "Veuillez entrer une valeur correcte pour le module sortie 1");
        return; // Arrêt de la méthode
    }
       
 
       if ((comboBoxctin1->currentText().isEmpty()) || (comboBoxctin2->currentText().isEmpty()) || (comboBoxctin3->currentText().isEmpty()) || (comboBoxctin4->currentText().isEmpty()) || (comboBoxctin_i->currentText().isEmpty()))
        {
    QMessageBox::critical(this, "Erreur", "Il y a au moins un module d'entree non definie: entrer la valeur manquante SVP!!!!");
        return; // Arrêt de la méthode
    }

       else if ((comboBoxctin3->currentText().isEmpty()) || (comboBoxctin4->currentText().isEmpty()))
        {
    QMessageBox::critical(this, "Erreur", "Il y a au moins un module de sortie non definie: entrer la valeur manquante SVP!");
        return; // Arrêt de la méthode
    }
       else if (comboBoxctin_i->currentText().isEmpty())
        {
    QMessageBox::critical(this, "Erreur", "La valeur de l'axe i est vide: entrer la valeur manquante SVP!");
        return; // Arrêt de la méthode
    }

if (l >= 0)
{
     

      tableWidgetctin1->setItem(row, 0, new QTableWidgetItem(comboBoxctin1->currentText()));
      tableWidgetctin1->setItem(row, 1, new QTableWidgetItem(comboBoxctin2->currentText()));
      tableWidgetctin1->setItem(row, 2, new QTableWidgetItem(comboBoxctin4->currentText()));
      tableWidgetctin1->setItem(row, 3, new QTableWidgetItem(comboBoxctin3->currentText()));
      tableWidgetctin1->setItem(row, 4, new QTableWidgetItem(comboBoxctin_i->currentText()));
      tableWidgetctin1->setItem(row, 5, new QTableWidgetItem(comboBoxctin_j->currentText()));
      tableWidgetctin1->setItem(row, 6, new QTableWidgetItem(comboBoxctin_k->currentText()));

          CtinData.push_back(comboBoxctin1->currentText() + " " + comboBoxctin2->currentText() + texte + comboBoxctin4->currentText() +
                                                          " " + comboBoxctin3->currentText() + " " + comboBoxctin_i->currentText() +
                                                          " " + comboBoxctin_j->currentText() + " " + comboBoxctin_k->currentText() +
                                                          " " + comboBoxctin_t->currentText()); 
    row++;
//}
        }
//writefileCtin();

*/
}
/********************************************** Implementation of the deleteCtinRow() method  *********************************************************/
void ConnectionWindow::deleteCtinRow() 
{
	try
		{
		}

	catch (...)
		{
			QMessageBox::critical(this, "Erreur", "erreur dans la suppression d'une ligne dans deleteCtinRow().");
		}
	


}

/******************************* Implementation of the activateInIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateInIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinIn2Combo->setEnabled(true);
			m_ctinIn3Combo->setEnabled(true);
			m_ctinFromInLabel->setEnabled(true);
			m_ctinToInLabel->setEnabled(true);
		}
	else
		{
			m_ctinIn2Combo->setEnabled(false);
			m_ctinIn3Combo->setEnabled(false);
			m_ctinFromInLabel->setEnabled(false);
			m_ctinToInLabel->setEnabled(false);
			
			m_ctinIn2Combo->setCurrentIndex(0);	
			m_ctinIn3Combo->setCurrentIndex(0);	
		}
	//m_ctinIngroupBox->setEnabled(true);	
}

/******************************* Implementation of the activateOutIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateOutIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinOut2Combo->setEnabled(true);
			m_ctinOut3Combo->setEnabled(true);
			m_ctinFromOutLabel->setEnabled(true);
			m_ctinToOutLabel->setEnabled(true);
		}
	else
		{
			m_ctinOut2Combo->setEnabled(false);
			m_ctinOut3Combo->setEnabled(false);
			m_ctinFromOutLabel->setEnabled(false);
			m_ctinToOutLabel->setEnabled(false);
			
			m_ctinOut2Combo->setCurrentIndex(0);	
			m_ctinOut3Combo->setCurrentIndex(0);	
	
		}
	//m_ctinIngroupBox->setEnabled(true);	
}
/******************************* Implementation of the activateIIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateIIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinI2Combo->setEnabled(true);
			m_ctinI3Combo->setEnabled(true);
			m_ctinFromILabel->setEnabled(true);
			m_ctinToILabel->setEnabled(true);
			m_ctinICheckBox->setEnabled(true);
		}
	else
		{
			m_ctinI2Combo->setEnabled(false);
			m_ctinI3Combo->setEnabled(false);
			m_ctinFromILabel->setEnabled(false);
			m_ctinToILabel->setEnabled(false);
			m_ctinICheckBox->setEnabled(false);
			
			m_ctinI2Combo->setCurrentIndex(0);	
			m_ctinI3Combo->setCurrentIndex(0);	
			m_ctinICheckBox->setChecked(false);	
		}
}
/******************************* Implementation of the activateJIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateJIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinJ2Combo->setEnabled(true);
			m_ctinJ3Combo->setEnabled(true);
			m_ctinFromJLabel->setEnabled(true);
			m_ctinToJLabel->setEnabled(true);
			m_ctinJCheckBox->setEnabled(true);
		}
	else
		{
			m_ctinJ2Combo->setEnabled(false);
			m_ctinJ3Combo->setEnabled(false);
			m_ctinFromJLabel->setEnabled(false);
			m_ctinToJLabel->setEnabled(false);
			m_ctinJCheckBox->setEnabled(false);
			
			m_ctinJ2Combo->setCurrentIndex(0);	
			m_ctinJ3Combo->setCurrentIndex(0);	
			m_ctinJCheckBox->setChecked(false);	
		}
}

/******************************* Implementation of the activateKIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateKIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinK2Combo->setEnabled(true);
			m_ctinK3Combo->setEnabled(true);
			m_ctinFromKLabel->setEnabled(true);
			m_ctinToKLabel->setEnabled(true);
			m_ctinKCheckBox->setEnabled(true);
		}
	else
		{
			m_ctinK2Combo->setEnabled(false);
			m_ctinK3Combo->setEnabled(false);
			m_ctinFromKLabel->setEnabled(false);
			m_ctinToKLabel->setEnabled(false);
			m_ctinKCheckBox->setEnabled(false);
			
			m_ctinK2Combo->setCurrentIndex(0);	
			m_ctinK3Combo->setCurrentIndex(0);	
			m_ctinKCheckBox->setChecked(false);	
		}
}

/******************************* Implementation of the activateTIntervalle(QString aString) method **********************************/
void ConnectionWindow::activateTIntervalle(QString aString ) 
{

	if (aString == "...")
		{
			m_ctinT2Combo->setEnabled(true);
			m_ctinT3Combo->setEnabled(true);
			m_ctinFromTLabel->setEnabled(true);
			m_ctinToTLabel->setEnabled(true);
			m_ctinTCheckBox->setEnabled(true);
		}
	else
		{
			m_ctinT2Combo->setEnabled(false);
			m_ctinT3Combo->setEnabled(false);
			m_ctinFromTLabel->setEnabled(false);
			m_ctinToTLabel->setEnabled(false);
			m_ctinTCheckBox->setEnabled(false);
			
			m_ctinT2Combo->setCurrentIndex(0);	
			m_ctinT3Combo->setCurrentIndex(0);	
			m_ctinTCheckBox->setChecked(false);	
		}
}

/****************************************************************************************************************************/
/*QStringList ConnectionWindow::getDataCtin() {

     return CtinData;
}*/

/****************************************************************************************/
void ConnectionWindow::toInitialize() 
{
	m_ctinModulInCombo->setCurrentIndex(0);
	m_ctinModulOutCombo->setCurrentIndex(0);
	m_ctinInCombo->setCurrentIndex(0);
	m_ctinOutCombo->setCurrentIndex(0);
	m_ctinICombo->setCurrentIndex(0);
	m_ctinJCombo->setCurrentIndex(0);
	m_ctinKCombo->setCurrentIndex(0);
	m_ctinTCombo->setCurrentIndex(0);
	m_ctinTCombo->setCurrentIndex(0);
	
	m_ctinIRadioButton->setChecked(false);
	m_ctinJRadioButton->setChecked(false);
	m_ctinKRadioButton->setChecked(false);
	m_ctinTRadioButton->setChecked(false);
		

	m_ctinJ2Combo->setCurrentIndex(0);
	m_ctinJ3Combo->setCurrentIndex(0);

	m_ctinK2Combo->setCurrentIndex(0);
	m_ctinK3Combo->setCurrentIndex(0);

	m_ctinT2Combo->setCurrentIndex(0);
	m_ctinT3Combo->setCurrentIndex(0);

	m_ctinIn2Combo->setCurrentIndex(0);	
	m_ctinIn3Combo->setCurrentIndex(0);	

	m_ctinOut2Combo->setCurrentIndex(0);	
	m_ctinOut3Combo->setCurrentIndex(0);	
	
	m_ctinICheckBox->setChecked(false);
	m_ctinJCheckBox->setChecked(false);
	m_ctinKCheckBox->setChecked(false);
	m_ctinTCheckBox->setChecked(false);

}

/********************************************* Implementation of ctinHelp() method *********************************************************************/
void ConnectionWindow::ctinHelp() {

	QMessageBox::about(this, tr("Aide sur la directive Ctin."),("<p> L’objet de la connectivité est d’établir l’échange d’information entre modules.<br> Chaque module possèdant un certain nombre d’entrées et de sorties; les modules sont distribués dans l’espace-temps répérable par des coordonnées (i, j, k, t).<p> La connectivité doit donc permettre d’indiquer les liens entre les différentes entrées et sorties des modules selon leur point de coordonnée (notion de voisinage). <p> <u>Exemple:</u> modul Hfil space Soce input 3 output 1 tempo cout target    "
                               "<p>Pour plus d'informations sur la directive ctin,  <a href=\"help/ctin_aide.html\">cliquez ici.</a>"));


}

/**************************************************************************************************************************************************/
void ConnectionWindow::closeEvent(QCloseEvent *event) {


	emit ctinWindowIsClosed(9);

}
/**********************************************************************************************************************************************/
/****************************************************** End of file. **************************************************************************/
/**********************************************************************************************************************************************/
