//! \file    ModulWindow.cpp
//! \brief   Implementation of the Modul window for visual_YAO.
//! \version 2010/07/05 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.

/*
  Nom du Fichier: ModulWindow.cpp
  Nom de classe:
  Role:
  Nombre de methodes:
  Nombre d'attributs:
  */

#include "ModulWindow.hpp"

using namespace std;
using namespace yao;

//! \class ModulWindow(QWidget *parent) : QWidget(parent)
//! \brief Implementation of the Modul window for visual_YAO.
//ModulWindow::ModulWindow(QWidget *parent):QWidget(parent) {
ModulWindow::ModulWindow(Table <Modul> *aModulTable, Table <Space> *aSpaceTable, Table <Operator> *anOperatorTable, Table <Constant> *aConstantTable, MainWindowController *mwc, string *aProjectName, string *aProjectPath, QWidget *parent) : QWidget(parent), m_localModulTable(NULL)/*, Modul(m_localModulName)*/ {


	setupUi(this);
	localConstantTable	= aConstantTable;
	m_localModulTable	= aModulTable;
	m_localSpaceTable	= aSpaceTable;
	m_localOperaTable	= anOperatorTable;
	m_localProjectName	= aProjectName;
	m_localProjectPath	= aProjectPath;
	

	QRegExp regExp("[A-Za-z_][A-Za-z1-9_]{0,}"); 
	m_modulNameLine->setValidator(new QRegExpValidator(regExp, this));
	//Modul(&m_localModulName);

	m_modulOperaCombo->setDisabled(true);
	m_modulTargetCombo->insertItem(0, " ");
	m_modulTargetCombo->insertItem(1, "uptime");
	m_modulTargetCombo->insertItem(2, "steptime");
	m_modulTargetCombo->insertItem(3, "alltime");

	m_modulInter2Combo->setDisabled(true);
	m_modulInter3Combo->setDisabled(true);
	
	//cout << "voici le chemin du projet courant: " << endl << *m_localProjectPath + "/" + *m_localProjectName << endl;
	
	if (isModulTableEmpty()) 
		m_modulDelButton->setDisabled(true);	
	//m_modulArrayCheckBox->setDisabled(true);
//	for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++)  {
//				cout << it->getName().c_str() << endl;
		//m_modulSpaceComboName->insertItem(2, "DOUBLE"); 
//		m_modulAutonetCombo->insertItem(2, QString(it->getName().c_str()));
		//m_comboModulClonof->insertItem(1, QString(it->().c_str()));
//		m_comboModulLopera->insertItem(1, QString(it->getName().c_str())); 

	m_modulSpaceCombo->insertItem(0, " ");
	m_modulOperaCombo->insertItem(0, " ");

	for (Table<Space>::reverse_iterator it = m_localSpaceTable->rbegin(); it != m_localSpaceTable->rend(); it++)  
		m_modulSpaceCombo->insertItem(1, QString(it->getProperty(0).c_str()));
		
	for (Table<Space>::reverse_iterator it = m_localOperaTable->rbegin(); it != m_localOperaTable->rend(); it++)  
		m_modulOperaCombo->insertItem(1, QString(it->getProperty(0).c_str()));
		
	m_modulModel 	= new QStandardItemModel(0, 12);
        	m_modulModel->setHorizontalHeaderLabels(QStringList() << "Nom module" << "space/opera" << "input" << "output" << "target" << "noward" << "autonet" << "netward" << "spec" << "cout" << "lopera" << "tempo");
			
		m_modulTableView->setModel(m_modulModel);
		m_modulTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
		m_modulTableView->setShowGrid(false);
		m_modulTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
		m_modulTableView->resizeColumnsToContents();

	for (Table<Modul>::iterator it = m_localModulTable->begin(); it != m_localModulTable->end(); it++) {
 
             int row = m_modulModel->rowCount();
 
             m_modulModel->setItem(row, 0, new QStandardItem(QString(it->getName().c_str()))); /*!< Gets the name.*/
             m_modulModel->setItem(row, 1, new QStandardItem(QString(it->getSpaceOrOperator().c_str()))); /*!< Gets the dimension (1, 2 or 3).*/
             //m_modulModel->setItem(row, 2, new QStandardItem(QString(it->getProperty(2).c_str()))); /*!< Gets the  choosed in the space directive .*/
           }


	if (m_modulInputLine->text().isEmpty())
		m_modulArrayCheckBox->setDisabled(true);
	//connect(m_modulInputLine, SIGNAL(textEdited(QString )), this, SLOT(close()));
	
	connect(m_modulQuitButton, SIGNAL(clicked()), this, SLOT(close()));
        connect(m_modulAddButton, SIGNAL(clicked()), this, SLOT(addModulRow()));
	connect(m_modulDelButton, SIGNAL(clicked()), this, SLOT(deleteModulRow()));
	connect(m_modulHelpButton, SIGNAL(clicked()), this, SLOT(callModulHelp()));
	
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulSpecCheckBox, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulNetwardCheckBox, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_labelModulAutonet, SLOT(setDisabled(bool )));
	connect(m_modulNowardCheckBox, SIGNAL(toggled(bool )), m_modulAutonetCombo, SLOT(setDisabled(bool )));

	connect(m_modulSpaceRadioButton, SIGNAL(toggled(bool )), m_modulOperaCombo, SLOT(setDisabled(bool )));
	connect(m_modulOperaRadioButton, SIGNAL(toggled(bool )), m_modulSpaceCombo, SLOT(setDisabled(bool )));

	
        connect(m_buttonLoperaAdd, SIGNAL(clicked()), this, SLOT(addLoperaRow()));
	
	connect(m_buttonLoperaDel, SIGNAL(clicked()), this, SLOT(deleteLoperaRow()));

	setFixedWidth(baseSize().width() + 766);


}

/**************************************** Implémentation de la méthode addModulRow *****************************************************************/
void ModulWindow::addModulRow() {
try {
		string 	modulName 	= m_modulNameLine->text().toStdString() ,
		modulParent,
		input		= m_modulInputLine->text().toStdString(),
		output		= m_modulOutputLine->text().toStdString(),
		target		= m_modulTargetCombo->currentText().toStdString(),
		modulAttributs;
	
	int row = m_modulModel->rowCount();

	if ((m_modulSpaceRadioButton->isChecked()) && (!m_modulSpaceCombo->currentText().isEmpty()))
		modulParent = m_modulSpaceCombo->currentText().toStdString() ;
	else
		if ((m_modulOperaRadioButton->isChecked()) && (!m_modulOperaCombo->currentText().isEmpty()))
			modulParent = m_modulOperaCombo->currentText().toStdString() ;
	//input and output attribut.	
	if(!input.empty()) 
		{
			 
			modulAttributs = "input " + input;
			if(!output.empty())
				modulAttributs += "output " + output;
		}	
	// Target attribut.
	if(!target.empty())
                 {
                          
                         modulAttributs = "target " + target;
                 }               
	

	if (!(modulName.empty() || modulParent.empty() )) { // !(false) => true if there are data in the lineEDit.
	
		m_modulModel->setItem(row, 0, new QStandardItem(QString(modulName.c_str())));  
		m_modulModel->setItem(row, 1, new QStandardItem(QString(modulParent.c_str())));  
		if (!input.empty())
			{ 
				m_modulModel->setItem(row, 2, new QStandardItem(QString(input.c_str())));  
				if (!output.empty()) 
					m_modulModel->setItem(row, 3, new QStandardItem(QString(output.c_str())));
			}
		if (!(output.empty()) && (input.empty())) {
			QMessageBox::critical(this, "Erreur", "Veuillez préciser la valeur du champ \"input\".");
                        return; // Arrêt de la méthode
		} 

		if (!target.empty())
				m_modulModel->setItem(row, 4, new QStandardItem(QString(target.c_str())));  

		if(m_modulNowardCheckBox->isChecked())
			{
				m_modulModel->setItem(row, 5, new QStandardItem("oui"));
				m_modulModel->setItem(row, 6, new QStandardItem("non"));
				m_modulModel->setItem(row, 7, new QStandardItem("non"));
				m_modulModel->setItem(row, 8, new QStandardItem("non"));
			}	
		
		(m_modulSpaceRadioButton->isChecked())? m_localModulTable->push_back(Modul(modulName, m_localSpaceTable->find(modulParent), true)): m_localModulTable->push_back(Modul(modulName, m_localOperaTable->find(modulParent), false));
		
		if (!m_modulNowardCheckBox->isChecked ()) generateModulFile(modulName); 		
	}

	toInitialize();
	if (!isModulTableEmpty()) m_modulDelButton->setEnabled(true);	
  }
	catch (...) {
		
		cout << "Erreur dans la methode addModulRow" << endl;
	}   	
	toInitialize();
}



/**************************************** Implémentation de la méthode addLoperaRow() **************************************************************/
void ModulWindow::addLoperaRow() {

	m_loperaModel->appendRow(new QStandardItem(m_modulLoperaCombo->currentText())); 
	m_modulLoperaCombo->setCurrentIndex(0);
}

/**************************************** Implémentation de la méthode deleteLoperaRow() **********************************************************/
void ModulWindow::deleteLoperaRow() {

	QItemSelectionModel *selection = m_loperaListView->selectionModel(); 
	m_loperaIndex = selection->currentIndex(); 
	 if (!m_loperaIndex.isValid()) 
		return; 
	QVariant element = m_loperaModel->data(m_loperaIndex, Qt::DisplayRole);
	int r = m_loperaIndex.row();
	if (selection->hasSelection()) {
	 m_loperaModel->removeRows(m_loperaListView->currentIndex().row(), 1); 
	//localHeaderList->erase(localHeaderList->begin() + r);
	//if(isHeaderListEmpty()) m_delHatButton->setDisabled(true);
	}
}
/************************************* Implémentation de la méthode toInitialize() ***********************************************************/
void ModulWindow::toInitialize() {

	m_modulNameLine->setText("");
	m_modulInputLine->setText("");
	m_modulOutputLine->setText("");
	m_modulTargetCombo->setCurrentIndex(0);
	m_modulSpaceCombo->setCurrentIndex(0); 
	m_modulOperaCombo->setCurrentIndex(0); 
	m_modulAutonetCombo->setCurrentIndex(0);
	m_modulClonofCombo->setCurrentIndex(0); 
	m_modulLoperaCombo->setCurrentIndex(0);
	m_modulNowardCheckBox->setChecked(false);	
	m_modulTempoCheckBox->setChecked(false);
	m_modulCoutCheckBox->setChecked(false);
	m_modulSpecCheckBox->setChecked(false);
	m_modulNetwardCheckBox->setChecked(false);
	m_modulHidjacCheckBox->setChecked(false);
	m_modulManageCheckBox->setChecked(false);
	m_modulArrayCheckBox->setChecked(false);
		
}
/*************************************** Implémentation de la méthode deleteModulRow ****************************************************************/
void ModulWindow::deleteModulRow() {
	try 
		{

/*			
           if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
            {
                   modeleSpace->removeRows(m_spaceTableView->currentIndex().row(), 1);
                   localSpaceTable->erase(localSpaceTable->begin() + r);
 
           }
 
         if (isSpaceTableEmpty()) m_spaceDeleteButton->setDisabled(true);
*/
			//QItemSelectionModel *selection = m_modulTableView->selectionModel();
			//QModelIndex indexElementSelectionne = selection->currentIndex();
	
			//int c = m_tableView->currentIndex().row();//renvoie le numero de la ligne selectionnée.
			QItemSelectionModel *selection = m_modulTableView->selectionModel();
			index = selection->currentIndex();
			QVariant element = m_modulModel->data(index, Qt::DisplayRole);
			int r = index.row();
			if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
				 {   
					m_modulModel->removeRows(m_modulTableView->currentIndex().row(), 1); 
					m_localModulTable->erase(m_localModulTable->begin() + r); 
    
					/*for (unsigned int i=0; i < localHeaderList->size(); i++ ) //Parcourt le vecteur de HatName en fonction de sa taille (nombre d'éléments)
						{   
							//localConstantTable->find(hatFile)->getText(); 
							if (element.toString().toStdString() == localHeaderList->at(i)) //Teste si l'élément à supprimer dans defval est aussi dans hat
								localHeaderList->erase(localHeaderList->begin() + i);//localHeaderList->begin() +i );   
						}   
					    */
					//emit rowIsDeleted(element.toString()); // signal emis lorsqu'une ligne est retirée du tableau. 
					if(isModulTableEmpty()) m_modulDelButton->setDisabled(true);
				}   
		}   

	catch(...) 
		{

			QMessageBox::critical(this, "Erreur", "erreur dans la suppression d'une ligne dans deleteModulRow().");
		}    

    //	QVariant elementSelectionne = filesListModele->data(indexElementSelectionne, Qt::DisplayRole);

     	//filesList.removeOne(elementSelectionne.toString());//Retrait de l'élément sélectionné de l'objet QStringList
     	//filesListModele->setStringList(filesList); //On indique au modèle la liste des éléments à afficher.
	
}

/*****************************************************************************************************************************/
void ModulWindow::saveModulData() {
	
}
/**********************************************************************/
bool ModulWindow::isModulTableEmpty() {

	if (m_localModulTable->empty()) 
		return true; 
	else                                                              
		return false;   
	
}
/*****************************************************************************************************************************/
/*void ModulWindow::enregistrerModul() {

	  //projetModul = new QFileDialog(this);
	  QStringList filtres;

	  //projetModul->setDirectory(".");
	  //projetModul->setFileMode (QFileDialog::AnyFile);
	  filtres << "*.h"
		  << "Tous les fichiers (*.*)";
	  //projetModul->setFilters(filtres);

	  //if(fd->exec()) {
	    //m_cheminProjetModul = projetModul->getSaveFileName();

}*/
/*****************************************************************************************************************************/
/*void ModulWindow::ouvrirProjetExistant3() {

	    //projetModul = new QFileDialog(this);
	  QStringList filtres;

	  //projetModul->setDirectory(".");
	  //projetModul->setFileMode (QFileDialog::AnyFile);
	  filtres << "*.h"
          << "Tous les fichiers (*.*)";
	  projetModul->setFilters(filtres);

	    m_cheminProjetModul = projetModul->getSaveFileName();
 }
*/
/****************************************************************************************************************************/
/*QStringList ModulWindow::getModulData() {

	//return filesList;
}
*/
/************************************ Implementation of generateModulFile(string aFilename) method. *************************************************/
void ModulWindow::generateModulFile(string aFilename) 
{
	date = new QDateEdit;
    		date->setDate(QDate::currentDate());
	//string theFile ;
	string path = *m_localProjectPath + "/" + *m_localProjectName + "/" + aFilename + ".h";
	ofstream theFile(path.c_str(), ios::out | ios::binary | ios::trunc);
		if(!theFile.is_open())
			throw ofstream::failure("couldn't create " + path);
	theFile << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "//		Date: " << date->date().toString().toStdString().c_str()    	   << endl; 
	theFile << "//===========================================================================" << endl;
	theFile << "forward ()									 " << endl;
	theFile << "{										 " << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << "}										 " << endl;
	theFile << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "// methodes classe : backward						 " << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "backward ()									 " << endl;
	theFile << "{										 " << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << endl;
	theFile << "}										 " << endl;
	theFile << "//===========================================================================" << endl;
	theFile << "//**************************  FIN DU MODULE   *******************************" << endl;

	theFile.close();
}

/****************************************************************************************************************************/
void ModulWindow::callModulHelp() {

	 QMessageBox::about(this, tr("Aide sur la directive Module."),("<p> Cette directive permet de déclarer les modules. Un module est identifié par son nom (modulname). Il doit ensuite être rattaché soit à un espace (introduit par le mot clé space suivi du nom de l’espace(spacename)) soit à un opérateur (introduit par le mot clé opera suivi du nom de l’opérateur (operaname)). <p> <u>Exemple:</u> modul Hfil space Soce input 3 output 1 tempo cout target"
                              "<p>Pour plus d'informations sur la directive module,  <a href=\"help/modul_aide.html\">cliquez ici.</a>"));

}

/********************************************************************/
void ModulWindow::closeEvent(QCloseEvent *event) {

	emit modulWindowIsClosed(8)	;
} 
