//! \file    SpaceWindow.cpp
//! \brief   Implementation of the Ctin window.
//! \version 2010/06/23 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane.
/*
  Nom du Fichier: SpaceWindow.cpp
  Nom de classe: SpaceWindow
  Role:
  Nombre de methodes:11
  Nombre d'attributs:8
  */

#include "include/SpaceWindow.hpp"
#include "include/TrajectoryWindow.hpp"
/*!
 * \class SpaceWindow
 * \brief The cpp file of the SpaceWindow class for visual_YAO.
 */

using namespace std;
using namespace yao;

/******************************************* Contructeur ***************************************************/
SpaceWindow::SpaceWindow(Table <Space> *aSpaceTable, Table <Trajectory> *aTrajectory, Table <Constant> *aConstantTable, QWidget *parent) : QWidget(parent), localTrajectoryTable(NULL), localSpaceTable(NULL) 
{       /*!< The constructor. */

	setupUi(this);

	localSpaceTable			= aSpaceTable;  /*!< See theSpaceTable in MainWindow.hpp */
	localTrajectoryTable		= aTrajectory;  /*!< See theTrajectoryTable in MainWindow.hpp */
	localConstantTable		= aConstantTable; /*!< See theConstantTable in MainWindow.hpp */
	
	QRegExp regExp("[A-Za-z_][A-Za-z0-9_]*");

	m_spaceNameLine->setValidator(new QRegExpValidator(regExp, this));

	if (isSpaceTableEmpty()) 
		m_spaceDeleteButton->setDisabled(true);/*!< To disable the m_spaceDeleteButton if theTrajectoryTable is empty. */
	
	m_spaceD1->setEditable(true);
	m_spaceD1->insertItem(0,""); 
	m_spaceD2->setEditable(true);
	m_spaceD2->insertItem(0,""); 
	m_spaceD3->setEditable(true);
	m_spaceD3->insertItem(0,""); 

	for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++) {

		m_spaceD1->insertItem(1, QString(it->getName().c_str()));
		m_spaceD2->insertItem(1, QString(it->getName().c_str()));
		m_spaceD3->insertItem(1, QString(it->getName().c_str()));
          }

	m_spaceTrajname->insertItem(0, "");

	for (Table<Trajectory>::reverse_iterator it = localTrajectoryTable->rbegin(); it != localTrajectoryTable->rend(); it++) 
		m_spaceTrajname->insertItem(1, QString(it->getName().c_str()));

	modeleSpace                    	= new QStandardItemModel(0, 3) ;
                modeleSpace->setHorizontalHeaderLabels(QStringList() << "Nom espace" << "Dimension(s) " << "Nom trajectoire"); //Names to the columns.

	m_spaceTableView->setModel(modeleSpace); /*!< The view takes model.*/
	m_spaceTableView->setEditTriggers(QAbstractItemView::NoEditTriggers); /*!< this ligne give no edit mode*/
	m_spaceTableView->setShowGrid(false);  /*!< this ligne cover the grille in table */
	m_spaceTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
		m_spaceTableView->resizeColumnsToContents(); /*!< To optimize the column space. */

	//The following to load in the visual Yao the structure of Yao.   
	for (Table<Space>::iterator it = localSpaceTable->begin(); it != localSpaceTable->end(); it++) {

	    int row = modeleSpace->rowCount();
            	
	    modeleSpace->setItem(row, 0, new QStandardItem(QString(it->getProperty(0).c_str()))); /*!< Gets the space name.*/
            modeleSpace->setItem(row, 1, new QStandardItem(QString(it->getProperty(1).c_str()))); /*!< Gets the space dimension (1, 2 or 3).*/
            modeleSpace->setItem(row, 2, new QStandardItem(QString(it->getProperty(2).c_str()))); /*!< Gets the trajectory choosed in the space directive.*/
          }


	connect(m_spaceAddButton, SIGNAL(clicked()), this, SLOT(addSpaceRow())); /*!< Adds rows in the tableview after clicking on the button "Add"*/
   	connect(m_spaceDeleteButton, SIGNAL(clicked()), this, SLOT(deleteRow()));
   	connect(m_spaceQuitButton, SIGNAL(clicked()), this, SLOT(close())); /*!< Will be closed after a click on the button "Quit" */
   	connect(m_spaceHelpButton, SIGNAL(clicked()), this, SLOT(callSpaceHelp())); /*!< Calls the help method.*/
    	
   	setFixedWidth(baseSize().width() + 505);
}
//------------------------------------  End of the contructor declaration.---------------------------------------
/*************************************  Implementation of function addSpaceRow *************************************************/
void SpaceWindow::addSpaceRow() {
	try 
		{

			int row = modeleSpace->rowCount();//To have the current row of the tableView.

			string name   	= m_spaceNameLine->text().toStdString(),
			d1   		= m_spaceD1->currentText().toStdString(),
			d2   		= m_spaceD2->currentText().toStdString(),
			d3   		= m_spaceD3->currentText().toStdString(),
			dim		,//= d1 + " " + d2 + " " + d3,
			traj 		= m_spaceTrajname->currentText().toStdString();
	
			vector <int> theShape; 	

			Constant *c1; //Nous créons d'abord un pointeur de constante.
			Constant *c2; //Nous créons d'abord un pointeur de constante.
			Constant *c3; //Nous créons d'abord un pointeur de constante.
 
			c1  = localConstantTable->find(d1); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
			c2 = localConstantTable->find(d2); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
			c3 = localConstantTable->find(d3); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.
	
			//Control on the first dimension's value. 
			if (c1 != 0) { //C'est alors une constante.

				dim = localConstantTable->find(d1)->getText();	
				theShape.push_back(toInt(localConstantTable->find(d1)->getText()));
		
			}
			else {

				for(unsigned int i=0; i < d1.size(); i++) 
					{
						if(!(d1[i] >= '0' && d1[i] <= '9')){
 
						     QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 1\"."));
						      return;
						 }

					}

				dim = d1;	
				theShape.push_back(toInt(d1));

			}
			//Control on the 2nd dimension's value. 
			if (c2 != NULL)  //C'est alors une constante.
				{
					dim =  dim + " " + localConstantTable->find(d2)->getText();	
					theShape.push_back(toInt(localConstantTable->find(d2)->getText()));
				}
			else 
				{
	
					for(unsigned int i=0; i < d2.size(); i++) {
						if(!(d2[i] >= '0' && d2[i] <= '9'))
							{
 
							     QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 2\"."));
							      return;
							 }

					}

					dim = dim + " " + d2;	
					theShape.push_back(toInt(d2));

				}

				//Control on the 3rd dimension's value. 
				if (c3 != NULL) { //C'est alors une constante.
					dim =  dim + " " + localConstantTable->find(d3)->getText();	
					theShape.push_back(toInt(localConstantTable->find(d3)->getText()));
				}
				else 
					{
						for(unsigned int i=0; i < d3.size(); i++) 
							{
								if(!(d3[i] >= '0' && d3[i] <= '9')){
									QMessageBox::about(this, tr("Erreur de saisie!"), tr("Saisir un entier ou faire un choix dans la liste deroulante du champ \"Dimension 3\"."));
 
									return;
								 }

							}

						dim = dim + " " + d3;	
						theShape.push_back(toInt(d3));
					}
 
				if (name.empty()) { 
					QMessageBox::about(this, tr("Erreur de saisie!"),
					     tr("<p>Le champ \"Nom espace\" ne doit pas etre vide."));
					return;
				}
				else
 
					 if (!(dim.empty() || traj.empty())) {
						modeleSpace->setItem(row, 0, new QStandardItem(m_spaceNameLine->text()));
						modeleSpace->setItem(row, 1, new QStandardItem(QString(dim.c_str())));
						modeleSpace->setItem(row, 2, new QStandardItem(m_spaceTrajname->currentText()));
		
						if (!(d1.empty() || d2.empty() || d3.empty())) {
							Space espace(name, localTrajectoryTable->find(traj), theShape);
							localSpaceTable->push_back(espace);	
						}

						toInitialize();
 
						if (!isSpaceTableEmpty()) m_spaceDeleteButton->setEnabled(true);
					}
		}

		catch(...)
			{
				QMessageBox::critical(this, "Erreur", "erreur dans la methode addSpaceRow().");
			}
}

/***********************************************************************************************************************/
const int SpaceWindow::toInt(const string& s) {
 
      return atoi(s.c_str());
}

/************************************* Implementation of function addRow **************************************************/
//void SpaceWindow::addRow() {

	//int row = m_spaceTableView->rowCount();
	//m_spaceTableView->insertRow(row);

//}

/************************************** Implementation of function deleteRow **********************************************/
void SpaceWindow::deleteRow() {

    	//m_spaceTableView->removeRow(m_spaceTableView->currentRow());
    	
	QItemSelectionModel *selection = m_spaceTableView->selectionModel();
	index = selection->currentIndex();
	QVariant element = modeleSpace->data(index, Qt::DisplayRole);
	int r = index.row();
	if (selection->hasSelection())// Permet de supprimer la ligne qui est selectionnée. Evite ainsi certains effets indesirables.
		{
			modeleSpace->removeRows(m_spaceTableView->currentIndex().row(), 1);
			localSpaceTable->erase(localSpaceTable->begin() + r);
		}
	
	if (isSpaceTableEmpty()) m_spaceDeleteButton->setDisabled(true);
}

/**********************************************************************/
bool SpaceWindow::isSpaceTableEmpty() { 

	if (localSpaceTable->empty())
		return true;
	else
		return false;
}
/************************************** Implementation of function saveInYaoStructure() ***********************************/
void SpaceWindow::saveInYaoStructure() {

	//for (QStringList::iterator it = m_localData.begin(); it != m_localData.end(); it++)
	//	m_spaceData.push_back(*it);
	m_spaceData = m_localData;
}

/****************************************************************************************************************************/

/****************************************************************************************************************************/
void SpaceWindow::toInitialize() {
	
	m_spaceNameLine->setText("");
	m_spaceD1->setCurrentIndex(0);
	m_spaceD2->setCurrentIndex(0);
	m_spaceD3->setCurrentIndex(0);
	m_spaceTrajname->setCurrentIndex(0);

}
/****************************************************************************************************************************/
bool SpaceWindow::isString(string aString) {
//if (aString.length() > 1) {
	bool stringTest = false;
	for (int cpt = 0; aString.size(); cpt++) 
		{ 
			if((aString[cpt] >= 'a' && aString[cpt] <= 'z') || (aString[cpt] >= 'A' && aString[cpt] <= 'Z') || (aString[cpt] >= '0' && aString[cpt] <= '9') || (aString[cpt] == '/') || (aString[cpt] == '_') || (aString[cpt] == '.') || (aString[cpt] == ' '))

				stringTest =  true;
			else
				stringTest =  false;        
		}
//}   
//else
//	{
//	QMessageBox::about(this, tr("Erreur de saisie!"),
  //                             tr("<p>Chaine trop courte!<p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));
    //         //return;
      //  }
	return stringTest;

}


/****************************************************************************************************************************/
bool SpaceWindow::isInt(string aStringToInt) {
 
	bool intTest = false;

	for (int cpt = 0; aStringToInt.size(); cpt++) 
		{ 
			if(aStringToInt[cpt] >= '0' && aStringToInt[cpt] <= '9')
				intTest = true;
			else
				intTest = false;        
		}
	
	return intTest;
}
/********************************************* Implementation of callSpaceHelp() (slot) *************************************************************/
void SpaceWindow::callSpaceHelp() 
{

	QMessageBox::about(this, tr("Aide sur la directive space."),("<p> La directive space permet de déclarer un espace qui sera identifié par son nom (spacename) et de sa taille qui peut être en 1, 2 ou 3 dimensions."
                                                                  "<p><u>Exemple 1</u>: space Soce M SZX SZY Toce <p> Pour plus d'informations sur la directive space, <a href=\"help/space_aide.html\">cliquez ici.</a>"));

}

/**************************************** Implementation of the method.******************************/
/**************************************** Implementation of the closeEvent(QCloseEvent *event) method. *******************************************/
void SpaceWindow::closeEvent(QCloseEvent *event) 
{

	emit spaceWindowIsClosed(5);
}
/***************************************************************************************************************************************************/
/*********************************************** End of file. **************************************************************************************/
/***************************************************************************************************************************************************/
